@@ -1,5 +1,5 @@
-# function creates a stochastic character mapped tree as a modified "phylo" object
-# written by Liam Revell 2013, 2014, 2015, 2016, 2017
+## function creates a stochastic character mapped tree as a modified "phylo" object
+## written by Liam Revell 2013, 2014, 2015, 2016, 2017, 2018, 2019
 
 make.simmap<-function(tree,x,model="SYM",nsim=1,...){
 	if(inherits(tree,"multiPhylo")){
@@ -42,9 +42,9 @@ make.simmap<-function(tree,x,model="SYM",nsim=1,...){
 		xx<-xx/rowSums(xx)
 		# reorder to cladewise
 		tree<-bt<-reorder.phylo(tree,"cladewise")
-		if(!is.binary.tree(bt)) bt<-multi2di(bt)
+		if(!is.binary(bt)) bt<-multi2di(bt)
 		# some preliminaries
-		N<-length(tree$tip)
+		N<-Ntip(tree)
 		m<-ncol(xx)
 		root<-N+1
 		# get conditional likelihoods & model
@@ -201,7 +201,7 @@ getPars<-function(bt,xx,model,Q,tree,tol,m,liks=TRUE,pi,args=list()){
 	if(liks){
 		L<-obj$lik.anc
 		rownames(L)<-N+1:nrow(L)
-		if(!is.binary.tree(tree)){
+		if(!is.binary(tree)){
 			ancNames<-matchNodes(tree,bt)
 			L<-L[as.character(ancNames[,2]),]
 			rownames(L)<-ancNames[,1]
@@ -238,7 +238,7 @@ smap<-function(tree,x,N,m,root,L,Q,pi,logL){
 	NN[which(tree$edge[,1]==root),1]<-rstate(L[as.character(root),]*pi/sum(L[as.character(root),]*pi)) # assign root
 	for(j in 1:nrow(tree$edge)){
 		# conditioned on the start value, assign end value of node (if internal)
-		p<-expm(Q*tree$edge.length[j])[NN[j,1],]*L[as.character(tree$edge[j,2]),]
+		p<-EXPM(Q*tree$edge.length[j])[NN[j,1],]*L[as.character(tree$edge[j,2]),]
 		NN[j,2]<-rstate(p/sum(p))
 		NN[which(tree$edge[,1]==tree$edge[j,2]),1]<-NN[j,2]
 		# now simulate on the branches
@@ -353,107 +353,147 @@ density.multiSimmap<-function(x,...){
 		if(hasArg(bw)) bw<-list(...)$bw
 		else bw<-1
 		tmp<-summary(x)
-		ab<-tmp$count[,2]
-		ba<-tmp$count[,3]
-		class(ab)<-class(ba)<-"mcmc"
+		ab<-lapply(2:ncol(tmp$count),function(i,x) x[,i],x=tmp$count)
+		names(ab)<-sapply(strsplit(colnames(tmp$count)[2:ncol(tmp$count)],
+			","),function(x) paste(x,collapse="->"))
+		ab<-lapply(ab,function(x){
+			class(x)<-"mcmc"
+			x })
 		if(.check.pkg("coda")){
-			hpd.ab<-HPDinterval(ab)
-			hpd.ba<-HPDinterval(ba)
+			hpd.ab<-lapply(ab,HPDinterval)
 		} else {
 			cat("  HPDinterval requires package coda.\n")
 			cat("  Computing 95% interval from samples only.\n\n")
-			hpd.ab<-c(sort(ab)[round(0.025*length(ab))],
-				sort(ab)[round(0.975*length(ab))])
-			hpd.ba<-c(sort(ba)[round(0.025*length(ba))],
-				sort(ba)[round(0.975*length(ba))])
+			hpd95<-function(x){
+				obj<-setNames(c(sort(x)[round(0.025*length(x))],
+					sort(x)[round(0.975*length(x))]),
+					c("lower","upper"))
+				attr(obj,"Probability")<-0.95
+				obj
+			}
+			hpd.ab<-lapply(ab,hpd95)
 		}
-		p.ab<-hist(ab,breaks=seq(min(c(ab,ba))-1.5,
-			max(c(ab,ba))+1.5,bw),plot=FALSE)
-		p.ba<-hist(ba,breaks=seq(min(c(ab,ba))-1.5,
-			max(c(ab,ba))+1.5,bw),plot=FALSE)
+		minmax<-range(unlist(ab))
+		pcalc<-function(x,mm)
+			hist(x,breaks=seq(mm[1]-1.5,mm[2]+1.5,bw),plot=FALSE)
+		p.ab<-lapply(ab,pcalc,mm=minmax)
 		states<-colnames(tmp$ace)
-		trans<-c(paste(states[1],"->",states[2],sep=""),
-			paste(states[2],"->",states[1],sep=""))
-		obj<-list(hpd.ab=hpd.ab,
-			hpd.ba=hpd.ba,
-			p.ab=p.ab,
-			p.ba=p.ba,
-			states=colnames(tmp$ace),trans=trans,
+		trans<-names(ab)
+		obj<-list(hpd=hpd.ab,
+			p=p.ab,
+			states=states,trans=trans,
 			bw=bw,
-			mins=setNames(c(min(ab),min(ba)),trans),
-			meds=setNames(c(median(ab),median(ba)),trans),
-			means=setNames(c(mean(ab),mean(ba)),trans),
-			maxs=setNames(c(max(ab),max(ba)),trans))
+			mins=sapply(ab,min),
+			meds=sapply(ab,median),
+			means=sapply(ab,mean),
+			maxs=sapply(ab,max))
 		class(obj)<-"changesMap"
 	}
+	else if(method=="timings"){
+		cat("This method doesn't work yet.\n")
+		obj<-NULL
+	}
 	obj
 }
 
+## S3 plot method for "changesMap" object from density.multiSimmap
 plot.changesMap<-function(x,...){
-	p.ab<-x$p.ab
-	p.ba<-x$p.ba
-	hpd.ab<-x$hpd.ab
-	hpd.ba<-x$hpd.ba
+	p<-x$p
+	hpd<-x$hpd
 	bw<-x$bw
-	plot(p.ab$mids,p.ab$density,xlim=c(min(x$mins)-1,
-		max(x$maxs)+1),ylim=c(0,1.2*max(c(p.ab$density,
-		p.ba$density))),
-		type="n",xlab="number of changes",
-		ylab="relative frequency across stochastic maps")
-	y2<-rep(p.ab$density,each=2)
-	y2<-y2[-length(y2)]
-	x2<-rep(p.ab$mids-bw/2,each=2)[-1]
-	x3<-c(min(x2),x2,max(x2))
-	y3<-c(0,y2,0)
-	polygon(x3,y3,col=make.transparent("red",0.3),border=FALSE)
-	lines(p.ab$mids-bw/2,p.ab$density,type="s")
-	y2<-rep(p.ba$density,each=2)
-	y2<-y2[-length(y2)]
-	x2<-rep(p.ba$mids-bw/2,each=2)[-1]
-	x3<-c(min(x2),x2,max(x2))
-	y3<-c(0,y2,0)
-	polygon(x3,y3,col=make.transparent("blue",0.3),border=FALSE)
-	lines(p.ba$mids-bw/2,p.ba$density,type="s")
-	add.simmap.legend(colors=setNames(c(make.transparent("red",0.3),
-		make.transparent("blue",0.3)),
-		c(paste(x$states[1],"->",x$states[2],sep=""),
-		paste(x$states[2],"->",x$states[1],sep=""))),
-		prompt=FALSE,x=min(x$mins),y=0.95*par()$usr[4])
-	dd<-0.01*diff(par()$usr[3:4])
-	lines(hpd.ab,rep(max(p.ab$density)+dd,2))
-	lines(rep(hpd.ab[1],2),c(max(p.ab$density)+dd,
-		max(p.ab$density)+dd-0.005))
-	lines(rep(hpd.ab[2],2),c(max(p.ab$density)+dd,
-		max(p.ab$density)+dd-0.005))
-	text(mean(hpd.ab),max(p.ab$density)+dd,
-		paste("HPD(",x$states[1],"->",x$states[2],")",sep=""),
-		pos=3)
-	lines(hpd.ba,rep(max(p.ba$density)+dd,2))
-	lines(rep(hpd.ba[1],2),c(max(p.ba$density)+dd,
-		max(p.ba$density)+dd-0.005))
-	lines(rep(hpd.ba[2],2),c(max(p.ba$density)+dd,
-		max(p.ba$density)+dd-0.005))
-	text(mean(hpd.ba),max(p.ba$density)+dd,
-		paste("HPD(",x$states[2],"->",x$states[1],")",sep=""),
-		pos=3)
+	if(length(x$trans)==2){
+		plot(p[[1]]$mids,p[[1]]$density,xlim=c(min(x$mins)-1,
+			max(x$maxs)+1),ylim=c(0,1.2*max(c(p[[1]]$density,
+			p[[2]]$density))),
+			type="n",xlab="number of changes",
+			ylab="relative frequency across stochastic maps")
+		y2<-rep(p[[1]]$density,each=2)
+		y2<-y2[-length(y2)]
+		x2<-rep(p[[1]]$mids-bw/2,each=2)[-1]
+		x3<-c(min(x2),x2,max(x2))
+		y3<-c(0,y2,0)
+		polygon(x3,y3,col=make.transparent("red",0.3),border=FALSE)
+		lines(p[[1]]$mids-bw/2,p[[1]]$density,type="s")
+		y2<-rep(p[[2]]$density,each=2)
+		y2<-y2[-length(y2)]
+		x2<-rep(p[[2]]$mids-bw/2,each=2)[-1]
+		x3<-c(min(x2),x2,max(x2))
+		y3<-c(0,y2,0)
+		polygon(x3,y3,col=make.transparent("blue",0.3),border=FALSE)
+		lines(p[[2]]$mids-bw/2,p[[2]]$density,type="s")
+		add.simmap.legend(colors=setNames(c(make.transparent("red",0.3),
+			make.transparent("blue",0.3)),x$trans[1:2]),
+			prompt=FALSE,x=min(x$mins),y=0.95*par()$usr[4])
+		dd<-0.01*diff(par()$usr[3:4])
+		lines(hpd[[1]],rep(max(p[[1]]$density)+dd,2))
+		lines(rep(hpd[[1]][1],2),c(max(p[[1]]$density)+dd,
+			max(p[[1]]$density)+dd-0.005))
+		lines(rep(hpd[[1]][2],2),c(max(p[[1]]$density)+dd,
+			max(p[[1]]$density)+dd-0.005))
+		text(mean(hpd[[1]]),max(p[[1]]$density)+dd,
+			paste("HPD(",x$trans[1],")",sep=""),
+			pos=3)
+		lines(hpd[[2]],rep(max(p[[2]]$density)+dd,2))
+		lines(rep(hpd[[2]][1],2),c(max(p[[2]]$density)+dd,
+			max(p[[2]]$density)+dd-0.005))
+		lines(rep(hpd[[2]][2],2),c(max(p[[2]]$density)+dd,
+			max(p[[2]]$density)+dd-0.005))
+		text(mean(hpd[[2]]),max(p[[2]]$density)+dd,
+			paste("HPD(",x$trans[2],")",sep=""),
+			pos=3)
+	} else {
+		k<-length(x$states)
+		par(mfrow=c(k,k))
+		ii<-1
+		max.d<-max(unlist(lapply(p,function(x) x$density)))
+		for(i in 1:k){
+			for(j in 1:k){
+				if(i==j) plot.new()
+				else {
+					plot(p[[ii]]$mids,p[[ii]]$density,xlim=c(min(x$mins)-1,
+						max(x$maxs)+1),ylim=c(0,1.2*max.d),
+						type="n",xlab="number of changes",
+						ylab="relative frequency",main=x$trans[ii],font.main=1)
+					##title(main=)
+					y2<-rep(p[[ii]]$density,each=2)
+					y2<-y2[-length(y2)]
+					x2<-rep(p[[ii]]$mids-bw/2,each=2)[-1]
+					x3<-c(min(x2),x2,max(x2))
+					y3<-c(0,y2,0)
+					polygon(x3,y3,col=make.transparent("blue",0.3),border=FALSE)
+					lines(p[[ii]]$mids-bw/2,p[[ii]]$density,type="s")
+					dd<-0.03*diff(par()$usr[3:4])
+					lines(hpd[[ii]],rep(max(p[[ii]]$density)+dd,2))
+					text(mean(hpd[[ii]]),max(p[[ii]]$density)+dd,"HPD",pos=3)
+					ii<-ii+1
+				}
+			}
+		}
+	}
 }
 
 print.changesMap<-function(x, ...){
 	if(hasArg(signif)) signif<-list(...)$signif
 	else signif<-2
 	cat("\nDistribution of changes from stochastic mapping:\n")
-	cat(paste("\t",x$trans[1],"\t\t",x$trans[2],"\n",sep=""))
-	cat(paste("\tMin.   :",round(x$mins[1],signif),
-		"\tMin.   :",round(x$mins[2],signif),"\n",sep=""))
-	cat(paste("\tMedian :",round(x$meds[1],signif),
-		"\tMedian :",round(x$meds[2],signif),"\n",sep=""))
-	cat(paste("\tMean   :",round(x$means[1],signif),
-		"\tMean   :",round(x$means[2],signif),"\n",sep=""))
-	cat(paste("\tMax.   :",round(x$maxs[1],signif),
-		"\tMax.   :",round(x$maxs[2],signif),"\n\n",sep=""))
-	cat("95% HPD interval(",x$trans[1],"): [",x$hpd.ab[1],", ",
-		x$hpd.ba[2],"]\n",sep="")
-	cat("95% HPD interval(",x$trans[2],"): [",x$hpd.ba[1],", ",
-		x$hpd.ba[2],"]\n\n",sep="")
+	NROW<-ceiling(length(x$trans)/2)
+	if(NROW>1) cat("\n")
+	for(i in 1:NROW){
+		ii<-2*i-1
+		jj<-ii+1
+		cat(paste("\t",x$trans[ii],"\t\t",x$trans[jj],"\n",sep=""))
+		cat(paste("\tMin.   :",round(x$mins[ii],signif),
+			"\tMin.   :",round(x$mins[jj],signif),"\n",sep=""))
+		cat(paste("\tMedian :",round(x$meds[ii],signif),
+			"\tMedian :",round(x$meds[jj],signif),"\n",sep=""))
+		cat(paste("\tMean   :",round(x$means[ii],signif),
+			"\tMean   :",round(x$means[jj],signif),"\n",sep=""))
+		cat(paste("\tMax.   :",round(x$maxs[ii],signif),
+			"\tMax.   :",round(x$maxs[jj],signif),"\n\n",sep=""))
+	}
+	for(i in 1:length(x$trans))
+		cat("95% HPD interval(",x$trans[i],"): [",x$hpd[[i]][1],", ",
+			x$hpd[[i]][2],"]\n",sep="")
+	cat("\n")
 }
 
