@@ -1,5 +1,77 @@
 ## some utility functions
-## written by Liam J. Revell 2011, 2012, 2013, 2014, 2015, 2016, 2017
+## written by Liam J. Revell 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
+
+## di2multi & multi2di for "contMap" & "densityMap" object classes
+
+di2multi.contMap<-function(phy,...){
+	phy$tree<-di2multi(phy$tree,...)
+	phy
+}
+
+di2multi.densityMap<-function(phy,...){
+	phy$tree<-di2multi(phy$tree,...)
+	phy
+}
+
+multi2di.contMap<-function(phy,...){
+	phy$tree<-multi2di(phy$tree,...)
+	phy
+}
+
+multi2di.densityMap<-function(phy,...){
+	phy$tree<-multi2di(phy$tree,...)
+	phy
+}
+
+## multi2di for "simmap" object class
+
+multi2di.simmap<-function(phy,...){
+	obj<-multi2di(as.phylo(phy),...)
+	M<-rbind(matchNodes(obj,phy),
+		matchLabels(obj,phy))
+	obj$maps<-vector(mode="list",length=nrow(obj$edge))
+	for(i in 2:nrow(M)){
+		if(!is.na(M[i,2])){
+			obj$maps[[which(obj$edge[,2]==M[i,1])]]<-
+				phy$maps[[which(phy$edge[,2]==M[i,2])]]
+		} else {
+			ii<-which(obj$edge[,2]==getParent(obj,M[i,1]))
+			state<-names(obj$maps[[ii]])[length(obj$maps[[ii]])]
+			obj$maps[[which(obj$edge[,2]==M[i,1])]]<-
+				setNames(0,state)
+		}
+	}
+	obj$node.states<-getStates(obj,"nodes")
+	obj$states<-getStates(obj,"tips")
+	obj$mapped.edge<-makeMappedEdge(obj$edge,obj$maps)
+	class(obj)<-c("simmap",class(obj))
+	obj
+}
+
+## di2multi & multi2di for "multiSimmap" object class
+
+di2multi.multiSimmap<-function(phy,...){
+	obj<-lapply(phy,di2multi,...)
+	class(obj)<-c("multiSimmap","multiPhylo")
+	obj
+}
+multi2di.multiSimmap<-function(phy,...){
+	obj<-lapply(phy,multi2di,...)
+	class(obj)<-c("multiSimmap","multiPhylo")
+	obj
+}
+
+## function to rescale a tree according to an EB model
+## written by Liam J. Revell 2017
+
+ebTree<-function(tree,r){
+	if(r!=0){
+		H<-nodeHeights(tree)
+		e<-(exp(r*H[,2])-exp(r*H[,1]))/r
+		tree$edge.length<-e
+	}
+	tree
+}
 
 ## function to expand clades in a plot by a given factor
 ## written by Liam J. Revell 2017
@@ -42,7 +114,7 @@ plot.expand.clade<-function(x,...){
 }
 
 ## function to add a geological or other temporal legend to a plotted tree
-## written by Liam J. Revell 2017
+## written by Liam J. Revell 2017, 2019
 geo.legend<-function(leg=NULL,colors=NULL,alpha=0.2,...){
 	if(hasArg(cex)) cex<-list(...)$cex
 	else cex<-par()$cex
@@ -100,6 +172,12 @@ geo.legend<-function(leg=NULL,colors=NULL,alpha=0.2,...){
 	if(plot){	
 		y<-c(rep(0,2),rep(par()$usr[4],2))
 		ylabel<--1/25*obj$Ntip
+		if(obj$direction=="rightwards"){
+			old.usr<-par()$usr
+			h<-max(obj$xx)
+			new.xlim<-c(h-par()$usr[1],h-par()$usr[2])
+			par(usr=c(new.xlim,old.usr[3:4]))
+		} else old.usr<-par()$usr
 		for(i in 1:nrow(leg)){
 			strh<-strheight(rownames(leg)[i])
 			polygon(c(leg[i,1:2],leg[i,2:1]),y,
@@ -128,13 +206,74 @@ geo.legend<-function(leg=NULL,colors=NULL,alpha=0.2,...){
 				leg[i,2]),y=c(0,ylabel,par()$usr[3],
 				par()$usr[3],ylabel,0),
 				col=colors[rownames(leg)[i]],border=NA)
+			strh<-strh*get.asp()
 			text(x=mean(leg[i,])+
 				if(obj$direction=="leftwards") 0.12*strh else -0.12*strh,
 				y=ylabel,labels=rownames(leg)[i],
 				srt=90,adj=c(1,0.5),cex=cex)
 		}
 	}
-	invisible(list(leg=leg,colors=colors))
+	par(usr=old.usr)
+	object<-list(leg=leg,colors=colors[1:nrow(leg)])
+	class(object)<-"geo.legend"
+	invisible(object)
+}
+
+print.geo.legend<-function(x,...){
+	cat("A geological period legend:\n")
+	colnames(x$leg)<-c("start","end")
+	print(data.frame(x$leg,color=x$colors))
+	cat("\n")
+}
+
+geo.palette<-function(){
+	colors<-setNames(c(
+		rgb(255,242,127,255,maxColorValue=255),
+		rgb(255,230,25,255,maxColorValue=255),
+		rgb(253,154,82,255,maxColorValue=255),
+		rgb(127,198,78,255,maxColorValue=255),
+		rgb(52,178,201,255,maxColorValue=255),
+		rgb(129,43,146,255,maxColorValue=255),
+		rgb(240,64,40,255,maxColorValue=255),
+		rgb(103,165,153,255,maxColorValue=255),
+		rgb(203,140,55,255,maxColorValue=255),
+		rgb(179,225,182,255,maxColorValue=255),
+		rgb(0,146,112,255,maxColorValue=255),
+		rgb(127,160,86,255,maxColorValue=255),
+		rgb(247,67,112,255,maxColorValue=255)),
+		c("Quaternary","Neogene","Paleogene",
+		"Cretaceous","Jurassic","Triassic",
+		"Permian","Carboniferous","Devonian",
+		"Silurian","Ordovician","Cambrian",
+		"Precambrian"))
+	leg<-rbind(c(2.588,0),
+		c(23.03,2.588),
+		c(66.0,23.03),
+		c(145.0,66.0),
+		c(201.3,145.0),
+		c(252.17,201.3),
+		c(298.9,252.17),
+		c(358.9,298.9),
+		c(419.2,358.9),
+		c(443.8,419.2),
+		c(485.4,443.8),
+		c(541.0,485.4),
+		c(4600,541.0))
+	rownames(leg)<-c("Quaternary","Neogene","Paleogene",
+		"Cretaceous","Jurassic","Triassic",
+		"Permian","Carboniferous","Devonian",
+		"Silurian","Ordovician","Cambrian",
+		"Precambrian")
+	colnames(leg)<-c("start","end")
+	object<-list(period=leg,cols=colors)
+	class(object)<-"geo.palette"
+	object
+}
+
+print.geo.palette<-function(x,...){
+	cat("A geological period color palette:\n")
+	print(data.frame(x$period,color=x$cols))
+	cat("\n")
 }
 
 ## borrowed from mapplots
@@ -198,7 +337,8 @@ linklabels<-function(text,tips,link.type=c("bent","curved","straight"),
 	if(link.type=="curved"){
 		for(i in 1:length(tips))
 			drawCurve(c(xpos[i],xmax[i]),c(ypos[i],ylab[i]),
-				scale=0.05,lty=lty,col=col,lwd=lwd)
+				scale=0.01*diff(range(lastPP$xx)),lty=lty,
+				col=col,lwd=lwd)
 	} else if(link.type=="bent"){
 		tipmax<-max(lastPP$xx)
 		for(i in 1:length(tips)){
@@ -418,7 +558,7 @@ modified.Grafen<-function(tree,power=2){
 ## function to compute all rotations
 ## written by Liam J. Revell 2016
 allRotations<-function(tree){
-	if(!is.binary.tree(tree)){
+	if(!is.binary(tree)){
 		was.binary<-FALSE
 		if(is.null(tree$edge.length)){ 
 			tree<-compute.brlen(tree)
@@ -642,18 +782,19 @@ fastDist<-function(tree,sp1,sp2){
 		2*fastHeight(tree,sp1,sp2)
 }
 
-# function reorders simmap tree
-# written Liam Revell 2011, 2013, 2015
+## function reorders simmap tree
+## written Liam Revell 2011, 2013, 2015, 2019
 reorderSimmap<-function(tree,order="cladewise",index.only=FALSE,...){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
 	ii<-reorder.phylo(tree,order,index.only=TRUE,...)
 	if(!index.only){
-		if(inherits(ii,"phylo")) ii<-whichorder(ii$edge[,2],tree$edge[,2]) ## bug workaround
+		if(inherits(ii,"phylo")) ii<-whichorder(ii$edge[,2],
+			tree$edge[,2]) ## bug workaround
 		tree$edge<-tree$edge[ii,]
 		tree$edge.length<-tree$edge.length[ii]
 		if(!is.null(tree$maps)){
 			tree$maps<-tree$maps[ii]
-			tree$mapped.edge<-tree$mapped.edge[ii,]
+			tree$mapped.edge<-tree$mapped.edge[ii,,drop=FALSE]
 		}
 		attr(tree,"order")<-order
 		return(tree)
@@ -739,7 +880,7 @@ labelSubTree<-function(tree,nn,label,pp,offset,wl,cex,orientation){
 	if(is.null(wl)) wl<-1
 	tree<-reorder(tree)
 	tips<-getDescendants(tree,nn)
-	tips<-tips[tips<=length(tree$tip.label)]
+	tips<-tips[tips<=Ntip(tree)]
 	ec<-0.7 ## expansion constant
 	sw<-pp$cex*max(strwidth(tree$tip.label[tips]))
 	sh<-pp$cex*max(strheight(tree$tip.label))
@@ -767,7 +908,7 @@ getExtant<-function(tree,tol=1e-8){
 	tl<-max(H)
 	x<-which(H[,2]>=(tl-tol))
 	y<-tree$edge[x,2]
-	y<-y[y<=length(tree$tip)]
+	y<-y[y<=Ntip(tree)]
 	z<-tree$tip.label[y]
 	return(z)
 }
@@ -779,7 +920,7 @@ getExtinct<-function(tree,tol=1e-8) setdiff(tree$tip.label,getExtant(tree,tol))
 
 splitTree<-function(tree,split){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
-	if(split$node>length(tree$tip.label)){
+	if(split$node>Ntip(tree)){
 		# first extract the clade given by shift$node
 		tr2<-extract.clade(tree,node=split$node)
 		tr2$root.edge<-tree$edge.length[which(tree$edge[,2]==split$node)]-split$bp
@@ -818,7 +959,7 @@ drop.clade<-function(tree,tip){
 
 
 ## function to re-root a phylogeny along an edge
-## written by Liam J. Revell 2011-2016
+## written by Liam J. Revell 2011-2016, 2019
 
 reroot<-function(tree,node.number,position=NULL,interactive=FALSE,...){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
@@ -835,7 +976,7 @@ reroot<-function(tree,node.number,position=NULL,interactive=FALSE,...){
 	p<-tt[[1]]
 	d<-tt[[2]]
 	tip<-if(length(which(p$tip.label=="NA"))>0) "NA" else p$tip.label[which(p$tip.label%in%tree$node.label)]
-	p<-root(p,outgroup=tip,resolve.root=T)
+	p<-ape::root.phylo(p,outgroup=tip,resolve.root=TRUE)
 	bb<-which(p$tip.label==tip)
 	p$tip.label[bb]<-"NA"
 	ee<-p$edge.length[which(p$edge[,2]==bb)]
@@ -859,7 +1000,7 @@ add.arrow<-function(tree=NULL,tip,...){
 	}
 	if(is.numeric(tip)){
 		ii<-tip
-		if(!is.null(tree)&&ii<=length(tree$tip.label)) tip<-tree$tip.label[ii]
+		if(!is.null(tree)&&ii<=Ntip(tree)) tip<-tree$tip.label[ii]
 		else tip<-""
 	} else if(is.character(tip)&&!is.null(tree)) ii<-which(tree$tip.label==tip)
 	if(hasArg(offset)) offset<-list(...)$offset
@@ -904,28 +1045,40 @@ add.arrow<-function(tree=NULL,tip,...){
 }
 
 ## function to ladderize phylogeny with mapped discrete character
-## written by Liam J. Revell 2014, 2015
+## written by Liam J. Revell 2014, 2015, 2019
 
 ladderize.simmap<-function(tree,right=TRUE){
-	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
-	obj<-read.tree(text=write.tree(ladderize(tree,right=right)))
-	rN<-Ntip(obj)+1
-	T<-cbind(1:Ntip(obj),sapply(obj$tip.label,function(x,y) which(y==x),y=tree$tip.label))
-	N<-matchNodes(obj,tree)
-	M<-rbind(T,N[N[,1]!=rN,])
-	ii<-sapply(M[,1],function(x,y) which(y==x),y=obj$edge[,2])
-	jj<-sapply(M[,2],function(x,y) which(y==x),y=tree$edge[,2])
-	obj$maps<-vector(length=nrow(tree$edge),mode="list")
-	obj$mapped.edge<-matrix(NA,nrow(tree$edge),ncol(tree$mapped.edge),
-		dimnames=list(apply(tree$edge,1,paste,collapse=","),
-		colnames(tree$mapped.edge)))
-	if(!is.null(tree$states)) 
-		obj$states<-tree$states[sapply(obj$tip.label,function(x,y) which(y==x),y=tree$tip.label)]
-	if(!is.null(tree$node.states)) obj$node.states<-matrix(NA,nrow(tree$edge),2)
-	for(i in 1:length(ii)){
-		obj$maps[[ii[i]]]<-tree$maps[[jj[i]]]
-		obj$mapped.edge[ii[i],]<-tree$mapped.edge[jj[i],]
-		if(!is.null(tree$node.states)) obj$node.states[ii[i],]<-tree$node.states[jj[i],]
+	if(!inherits(tree,"simmap")){
+		if(!inherits(tree,"phylo")) 
+			stop("tree should be an object of class \"phylo\".")
+		else {
+			cat("Do not detect a mapped character. Using ape::ladderize.\n")
+			obj<-ladderize(tree,right=right)
+		}
+	} else {
+		obj<-read.tree(text=write.tree(ladderize(tree,right=right)))
+		rN<-Ntip(obj)+1
+		T<-cbind(1:Ntip(obj),sapply(obj$tip.label,
+			function(x,y) which(y==x),y=tree$tip.label))
+		N<-matchNodes(obj,tree)
+		M<-rbind(T,N[N[,1]!=rN,])
+		ii<-sapply(M[,1],function(x,y) which(y==x),y=obj$edge[,2])
+		jj<-sapply(M[,2],function(x,y) which(y==x),y=tree$edge[,2])
+		obj$maps<-vector(length=nrow(tree$edge),mode="list")
+		obj$mapped.edge<-matrix(NA,nrow(tree$edge),ncol(tree$mapped.edge),
+			dimnames=list(apply(tree$edge,1,paste,collapse=","),
+			colnames(tree$mapped.edge)))
+		if(!is.null(tree$states)) 
+			obj$states<-tree$states[sapply(obj$tip.label,
+				function(x,y) which(y==x),y=tree$tip.label)]
+		if(!is.null(tree$node.states)) obj$node.states<-matrix(NA,nrow(tree$edge),2)
+		for(i in 1:length(ii)){
+			obj$maps[[ii[i]]]<-tree$maps[[jj[i]]]
+			obj$mapped.edge[ii[i],]<-tree$mapped.edge[jj[i],]
+			if(!is.null(tree$node.states)) obj$node.states[ii[i],]<-
+				tree$node.states[jj[i],]
+		}
+		class(obj)<-c("simmap","phylo")
 	}
 	obj
 }
@@ -1132,9 +1285,9 @@ nodeheight<-function(tree,node,...){
 	if(root.edge) ROOT<-if(!is.null(tree$root.edge)) tree$root.edge else 0
 	else ROOT<-0 
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
-	if(node==(length(tree$tip.label)+1)) h<-0
+	if(node==(Ntip(tree)+1)) h<-0
 	else {
-		a<-setdiff(c(getAncestors(tree,node),node),length(tree$tip.label)+1)
+		a<-setdiff(c(getAncestors(tree,node),node),Ntip(tree)+1)
 		h<-sum(tree$edge.length[sapply(a,function(x,e) which(e==x),e=tree$edge[,2])])
 	}
 	h+ROOT
@@ -1176,7 +1329,7 @@ getAncestors<-function(tree,node,type=c("all","parent")){
 	type<-type[1]
 	if(type=="all"){
 		aa<-vector()
-		rt<-length(tree$tip.label)+1
+		rt<-Ntip(tree)+1
 		currnode<-node
 		while(currnode!=rt){
 			currnode<-getAncestors(tree,currnode,"parent")
@@ -1222,7 +1375,6 @@ vcvPhylo<-function(tree,anc.nodes=TRUE,...){
 	}
 	if(hasArg(model)) model<-list(...)$model
 	else model<-"BM"
-
 	if(hasArg(tol)) tol<-list(...)$tol
 	else tol<-1e-12
 	if(model=="OU"){
@@ -1236,9 +1388,15 @@ vcvPhylo<-function(tree,anc.nodes=TRUE,...){
 			tree<-lambdaTree(tree,lambda)
 		} else model<-"BM"
 		model<-"BM"
-	}	
+	}
+	if(model=="EB"){
+		if(hasArg(r)){
+			r<-list(...)$r
+			tree<-ebTree(tree,r)
+		} else model<-"BM"
+	}
 	# done settings
-	n<-length(tree$tip.label)
+	n<-Ntip(tree)
 	h<-nodeHeights(tree)[order(tree$edge[,2]),2]
 	h<-c(h[1:n],0,h[(n+1):length(h)])
 	M<-mrca(tree,full=internal)[c(1:n,internal*(n+2:tree$Nnode)),c(1:n,internal*(n+2:tree$Nnode))]
@@ -1259,7 +1417,7 @@ vcvPhylo<-function(tree,anc.nodes=TRUE,...){
 ## simplified lambdaTree to be used internally by vcvPhylo
 ## written by Liam J. Revell 2014
 lambdaTree<-function(tree,lambda){
-	ii<-which(tree$edge[,2]>length(tree$tip.label))
+	ii<-which(tree$edge[,2]>Ntip(tree))
 	H1<-nodeHeights(tree)
 	tree$edge.length[ii]<-lambda*tree$edge.length[ii]
 	H2<-nodeHeights(tree)
@@ -1310,29 +1468,22 @@ di2multi.simmap<-function(phy,...){
 
 # returns the heights of each node
 # written by Liam J. Revell 2011, 2012, 2013, 2015, 2016
+# modified by Klaus Schliep 2017
 nodeHeights<-function(tree,...){
-	if(hasArg(root.edge)) root.edge<-list(...)$root.edge
-	else root.edge<-FALSE
-	if(root.edge) ROOT<-if(!is.null(tree$root.edge)) tree$root.edge else 0
-	else ROOT<-0 
-	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
-	if(attr(tree,"order")!="cladewise"||is.null(attr(tree,"order"))) t<-reorder(tree)
-	else t<-tree
-	root<-length(t$tip.label)+1
-	X<-matrix(NA,nrow(t$edge),2)
-	for(i in 1:nrow(t$edge)){
-		if(t$edge[i,1]==root){
-			X[i,1]<-0.0
-			X[i,2]<-t$edge.length[i]
-		} else {
-			X[i,1]<-X[match(t$edge[i,1],t$edge[,2]),2]
-			X[i,2]<-X[i,1]+t$edge.length[i]
-		}
-	}
-	if(attr(tree,"order")!="cladewise"||is.null(attr(tree,"order")))
-		o<-apply(matrix(tree$edge[,2]),1,function(x,y) which(x==y),y=t$edge[,2])
-	else o<-1:nrow(t$edge)
-	return(X[o,]+ROOT)
+    if(hasArg(root.edge)) root.edge<-list(...)$root.edge
+    else root.edge<-FALSE
+    if(root.edge) ROOT<-if(!is.null(tree$root.edge)) tree$root.edge else 0
+    else ROOT<-0 
+    nHeight <- function(tree){
+        tree <- reorder(tree)
+        edge <- tree$edge
+        el <- tree$edge.length
+        res <- numeric(max(tree$edge))
+        for(i in seq_len(nrow(edge))) res[edge[i,2]] <- res[edge[i,1]] + el[i] 
+        res
+    }
+    nh <- nHeight(tree)
+    return(matrix(nh[tree$edge], ncol=2L)+ROOT)
 }
 
 ## function drops all the leaves from the tree & collapses singleton nodes
@@ -1343,7 +1494,7 @@ drop.leaves<-function(tree,...){
 	if(hasArg(keep.tip.labels)) keep.tip.labels<-list(...)$keep.tip.labels
 	else keep.tip.labels<-FALSE
 	## end optional arguments
-	n<-length(tree$tip.label)
+	n<-Ntip(tree)
 	edge<-tree$edge
 	edge[edge>n]<--edge[edge>n]+n
 	ii<-which(edge[,2]>0)
@@ -1454,7 +1605,7 @@ mergeMappedStates<-function(tree,old.states,new.state){
 # written by Liam J. Revell 2013, 2015
 rotateNodes<-function(tree,nodes,polytom=c(1,2),...){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
-	n<-length(tree$tip.label)
+	n<-Ntip(tree)
 	if(nodes[1]=="all") nodes<-1:tree$Nnode+n
 	for(i in 1:length(nodes)) tree<-rotate(tree,nodes[i],polytom)
 	if(hasArg(reversible)) reversible<-list(...)$reversible
@@ -1497,8 +1648,8 @@ bind.tip<-function(tree,tip.label,edge.length=NULL,where=NULL,position=0,interac
 		obj<-get.treepos(message=FALSE)
 		where<-obj$where
 		position<-obj$pos
-	} else if(is.null(where)) where<-length(tree$tip.label)+1
-	if(where<=length(tree$tip.label)&&position==0){
+	} else if(is.null(where)) where<-Ntip(tree)+1
+	if(where<=Ntip(tree)&&position==0){
 		pp<-1e-12
 		if(tree$edge.length[which(tree$edge[,2]==where)]<=1e-12){
 			tree$edge.length[which(tree$edge[,2]==where)]<-2e-12
@@ -1507,7 +1658,7 @@ bind.tip<-function(tree,tip.label,edge.length=NULL,where=NULL,position=0,interac
 	} else pp<-position
 	if(is.null(edge.length)&&is.ultrametric(tree)){
 		H<-nodeHeights(tree)
-		if(where==(length(tree$tip.label)+1)) edge.length<-max(H)
+		if(where==(Ntip(tree)+1)) edge.length<-max(H)
 		else edge.length<-max(H)-H[tree$edge[,2]==where,2]+position
 	}
 	tip<-list(edge=matrix(c(2,1),1,2),
@@ -1516,7 +1667,7 @@ bind.tip<-function(tree,tip.label,edge.length=NULL,where=NULL,position=0,interac
 		Nnode=1)
 		class(tip)<-"phylo"
 	obj<-bind.tree(tree,tip,where=where,position=pp)
-	if(where<=length(tree$tip.label)&&position==0){
+	if(where<=Ntip(tree)&&position==0){
 		nn<-obj$edge[which(obj$edge[,2]==which(obj$tip.label==tip$tip.label)),1]
 		obj$edge.length[which(obj$edge[,2]==nn)]<-obj$edge.length[which(obj$edge[,2]==nn)]+1e-12
 		obj$edge.length[which(obj$edge[,2]==which(obj$tip.label==tip$tip.label))]<-0
@@ -1530,17 +1681,27 @@ bind.tip<-function(tree,tip.label,edge.length=NULL,where=NULL,position=0,interac
 	obj
 }
 
-# function collapses the subtree descended from node to a star tree
-# written by Liam J. Revell 2013, 2015
+## function collapses the subtree descended from node to a star tree
+## written by Liam J. Revell 2013, 2015, 2019
 collapse.to.star<-function(tree,node){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
-	nel<-if(is.null(tree$edge.length)) TRUE else FALSE
-	if(nel) tree$edge.length<-rep(1,nrow(tree$edge))
-	tt<-splitTree(tree,split=list(node=node,bp=tree$edge.length[which(tree$edge[,2]==node)]))
-	ss<-starTree(species=tt[[2]]$tip.label,branch.lengths=diag(vcv(tt[[2]])))
-	ss$root.edge<-0
-	tree<-paste.tree(tt[[1]],ss)
-	if(nel) tree$edge.length<-NULL 
+	if(node==(Ntip(tree)+1)){
+		object<-list(edge=cbind(rep(Ntip(tree)+1,Ntip(tree)),1:Ntip(tree)),
+			tip.label=tree$tip.label,Nnode=1)
+		if(!is.null(tree$edge.length)) object$edge.length<-sapply(1:Ntip(tree),nodeheight,tree=tree)
+		class(object)<-"phylo"
+		tree<-object
+	} else {
+		nel<-if(is.null(tree$edge.length)) TRUE else FALSE
+		if(nel) tree$edge.length<-rep(1,nrow(tree$edge))
+		tt<-splitTree(tree,split=list(node=node,
+			bp=tree$edge.length[which(tree$edge[,2]==node)]))
+		ss<-starTree(species=tt[[2]]$tip.label,
+			branch.lengths=diag(vcv(tt[[2]])))
+		ss$root.edge<-0
+		tree<-paste.tree(tt[[1]],ss)
+		if(nel) tree$edge.length<-NULL
+	}
 	tree
 }
 
@@ -1568,7 +1729,7 @@ findMRCA<-function(tree,tips=NULL,type=c("node","height")){
 extract.clade.simmap<-function(tree,node){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
 	x<-getDescendants(tree,node)
-	x<-x[x<=length(tree$tip.label)]
+	x<-x[x<=Ntip(tree)]
 	drop.tip.simmap(tree,tree$tip.label[-x])
 }
 
@@ -1576,11 +1737,11 @@ extract.clade.simmap<-function(tree,node){
 # written by Liam J. Revell 2013, 2015
 getCladesofSize<-function(tree,clade.size=2){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
-	n<-length(tree$tip.label)
+	n<-Ntip(tree)
 	nn<-1:(tree$Nnode+n)
 	ndesc<-function(tree,node){
 		x<-getDescendants(tree,node)
-		sum(x<=length(tree$tip.label))
+		sum(x<=Ntip(tree))
 	}
 	dd<-setNames(sapply(nn,ndesc,tree=tree),nn)
 	aa<-n+1 # root
@@ -1609,10 +1770,10 @@ getStates<-function(tree,type=c("nodes","tips")){
 	} else if(inherits(tree,"phylo")){ 
 		if(type=="nodes"){
 			y<-setNames(sapply(tree$maps,function(x) names(x)[1]),tree$edge[,1])
-			y<-y[as.character(length(tree$tip.label)+1:tree$Nnode)]
+			y<-y[as.character(Ntip(tree)+1:tree$Nnode)]
 		} else if(type=="tips"){
 			y<-setNames(sapply(tree$maps,function(x) names(x)[length(x)]),tree$edge[,2])
-			y<-setNames(y[as.character(1:length(tree$tip.label))],tree$tip.label)
+			y<-setNames(y[as.character(1:Ntip(tree))],tree$tip.label)
 		}
 	} else stop("tree should be an object of class \"phylo\" or \"multiPhylo\".")
 	return(y)
@@ -1784,7 +1945,7 @@ matchDatatoTree<-function(tree,x,name){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
 	if(is.matrix(x)) x<-x[,1]
 	if(is.null(names(x))){
-		if(length(x)==length(tree$tip.label)){
+		if(length(x)==Ntip(tree)){
 			print(paste(name,"has no names; assuming x is in the same order as tree$tip.label"))
 			names(x)<-tree$tip.label
 		} else
@@ -1869,7 +2030,7 @@ getSisters<-function(tree,node,mode=c("number","label")){
 	if(mode=="number") return(sisters)
 	else if(mode=="label"){
 		result<-list()
-		n<-length(tree$tip.label)
+		n<-Ntip(tree)
 		if(is.null(tree$node.label)&&any(sisters>n)) result$nodes<-sisters[which(sisters>n)] 
 		else if(any(sisters>n)) result$nodes<-tree$node.label[sisters[which(sisters>n)]-n]
 		if(any(sisters<=n)) result$tips<-tree$tip.label[sisters[which(sisters<=n)]]
@@ -1884,8 +2045,8 @@ getDescendants<-function(tree,node,curr=NULL){
 	if(is.null(curr)) curr<-vector()
 	daughters<-tree$edge[which(tree$edge[,1]==node),2]
 	curr<-c(curr,daughters)
-	if(length(curr)==0&&node<=length(tree$tip.label)) curr<-node
-	w<-which(daughters>length(tree$tip.label))
+	if(length(curr)==0&&node<=Ntip(tree)) curr<-node
+	w<-which(daughters>Ntip(tree))
 	if(length(w)>0) for(i in 1:length(w)) 
 		curr<-getDescendants(tree,daughters[w[i]],curr)
 	return(curr)
@@ -1931,13 +2092,6 @@ matchType<-function(type,types){
 			type=types[i]
 	return(type)
 }
-
-# wraps around MatrixExp
-# written by Liam Revell 2011
-expm<-function(Y){
-	Z<-MatrixExp(Y); dimnames(Z)<-dimnames(Y)
-	return(Z)
-}
 	
 # function 'untangles' (or attempts to untangle) a tree with crossing branches
 # written by Liam J. Revell 2013, 2015
